// -*-c++-*-
/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * Board.h: Board class implementing a tetris board.  
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __Board__
#define __Board__

#include <stdio.h>

#include <DisplayBox.h>

#include "Square.h"
#include "Pieces.h"

class Board : public DisplayBox {
public:
   // parameters are
   // x, y      -- the x/y coordinates of the upper left corner of the board.
   // w, h      -- the width and height in "squares" of the tetris board
   // sqw, sqh  -- the width and height of each square, in pixels
   // bg        -- the background color of the board.
   Board(int x, int y, int w, int h, int sqw, int sqh, int bg, 
         char *title = NULL);
   ~Board();

   // x/y are squares, not pixels. y can be (-4 <= y < Board Height).
   // -4 starts the piece falling off screen.
   void placePiece(const Piece& p, int orient, int x, int y, 
                   bool falling = true); 
   void erasePiece(const Piece& p, int orient, int x, int y);
   void setSquare(int x, int y, int color);
   void eraseSquare(int x, int y) { Squares[y + 4][x].setActive(false); }

   int eraseRows();		// erase all full rows, return number erased
   bool canPlace(const Piece& p, int orient, int x, int y); // true if piece will fit

   // redraw board squares which have changed since last update
   void update();		
   void touch();		// make it update every square
   void clear();		// clear the board

   int boardwidth() const { return BoardWidth; }
   int boardheight() const { return BoardHeight - 4; }
   
private:
   int BoardWidth;		// width and height in squares
   int BoardHeight;
   bool Touched;		// if true, update also redraws border
   
   Square **Squares;		// the board squares

   // The Board is actually 4 rows higher than is displayed, to allow pieces
   // to start falling from off the screen.  All the public functions add 4
   // to the 'y' parameter, to make this transperant from outside, but the
   // private functions expect this to already be adjusted for.
   
   // true if piece fits within board boundaries
   bool fits(const Piece& p, int orient, int x, int y);
   void eraseRow(int y);	// erase a row
};
   
   
inline void Board::setSquare(int x, int y, int color)
{
   y += 4;
   Squares[y][x].setActive(true);
   Squares[y][x].setFalling(false);
   Squares[y][x].setColor(color);
}

#endif // __Board__
