// -*-c++-*-
/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * Square.h: Class for the squares on the tetris board. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#ifndef __Square__
#define __Square__

class Square {
public:
   Square()
      {
	 Active = false;
	 Changed = true;
      }
   Square(int color, bool active = false)
      {
	 Color = color;
	 Active = active;
	 Changed = true;
      }


   void setActive(bool a) { Active = a; Changed = true; }
   void setColor(int c)  { Color = c; Changed = true; }
   void setFalling(bool f) { Falling = f; }
   void touch(bool c = true) { Changed = c; }
   void display(int x, int y);	// display square
   Square& operator = (Square& s); // assignment opterator
   // set static data members
   static void setBg(int color) { Bg = color; }
   static void setWidth(int w) { Width = w; }
   static void setHeight(int h) { Height = h; }  
   
   bool active() const { return Active; }
   // changed returns true if square has changed since last display call
   bool changed() const { return Changed; }
   bool falling() const { return Falling; }
   int color() const { return Color; }
   static int width() { return Width; }
   static int height() { return Height; }

   
private:
   int Color;
   bool Active;
   bool Changed;        // true if square has changed since last display call
   bool Falling;        // true if square is part of a falling piece
   
   // static class members
   static int Bg;	// color of a square when it isn't active
   static int Width;	// all squares have same width and height
   static int Height;
   static const int BORDER_DARK = 19;	// Border colors of squares
   static const int BORDER_LIGHT = 245;
};

inline Square& Square::operator = (Square& s)
{
   Color = s.Color;
   Active = s.Active;
   Changed = true;   // always true, don't just copy it
   Falling = s.Falling;
   return *this;
}

#endif  // __Square__
