// -*-c++-*-

/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * Button.h: Button class definition.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __Button__
#define __Button__

#include "Colors.h"

class Button {
public:
   Button (int x, int y, int w, char *text, int id = 0,
	   int fg = Clr::WHITE, int bg = Clr::GRAY);
   Button (int x, int y, char *text, int id = 0,
	   int fg = Clr::WHITE, int bg = Clr::GRAY);
   ~Button () { delete [] Text; }

   void display(bool active = false); // display button

   // display at new x/y. This also changes x/y for future calls
   void display(int x, int y, bool active = false); 

   void activate();		// display button as activated
   void deactivate();		// display button as deactivated
   bool inside(int x, int y);	// true if x/y are in the button's region
   
   int id() const { return Id; }
   int x1() const { return X1; }
   int x2() const { return X2; }
   int y1() const { return Y1; }
   int y2() const { return Y2; }
   int width() const { return (X2 - X1) + 1; }
   int height() const { return (Y2 - Y1) + 1; }
   
private:
   int X1;			// left x
   int Y1;			// top y
   int X2;			// right x
   int Y2;			// bottom y
   int Id;			// buttons Id
   char *Text;			// the button text
   int Fg;			// color of button text
   int Bg;			// background button color
   bool Active;			// is Button currently active
};

inline void Button::display(int x, int y, bool active)
{
   X2 = x + (X2 - X1);
   Y2 = y + (Y2 - Y1);
   X1 = x;
   Y1 = y;
   display(active);
}

inline void Button::activate()
{
   Active = true;
   display(true);
}

inline void Button::deactivate()
{
   Active = false;
   display(false);
}

inline bool Button::inside(int x, int y)
{
   return (x >= X1 && x <= X2 && y >= Y1 && y <= Y2) ? true : false;
}
#endif
