// -*-c++-*-

/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * CPalette.h: CPalette class definition.  Class to hold a palette.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
//////////////////////////////////////////////////////////////////////////
// 256 color Palette object 

#ifndef __CPalette__
#define __CPalette__

#include <sys/types.h>

class CPalette
{
public:
   CPalette() : Vpalette(NULL), Valloc(0)
      {};
   
   CPalette(int colors) 
      {
	 freePalette();
	 Vpalette = new Color [colors];
	 VpaletteByteSize = colors * 3;
	 VnumColors = colors;
	 Valloc = 1;
      } 

   ~CPalette() { freePalette(); }
   
   void	set(u_char *p, int size, int colors);
   void    set(CPalette& p); // sets this to a copy of p
   void    makeRGBPalette(void); // set this to a RGB palette
   void    setColor(int c, u_char r, u_char g, u_char b);
   
   void   getRGB(int c, u_char& r, u_char& g, u_char& b);
   
   // This is only meaningful if using the RGB palette 
   static int    getColor(u_char r, u_char g, u_char b)  
      {
	 return (r & 0xc0) + ((g & 0xe0) >> 2) + (b >> 5);
      }
   
   void releaseOwnership() { Valloc = 0; }

   int  Size() const { return VpaletteByteSize; }
   int  Colors() const { return VnumColors; }
   u_char *Pal(void) const { return (u_char *)Vpalette; }   

private:
   void	freePalette()
      {
	 if (Valloc) delete [] Vpalette;
	 Valloc=0;
      }
      
   typedef u_char Color[3];
   typedef Color  *ColorMap;

   static const char RED = 0;
   static const char GREEN = 1;
   static const char BLUE = 2;

   ColorMap Vpalette;
   char	 Valloc;
   int	 VpaletteByteSize;
   int	 VnumColors;

};

inline void CPalette::set(u_char *p, int size, int colors)
{
   freePalette();
   Vpalette = (ColorMap) p;
   VpaletteByteSize = size;
   VnumColors = colors;
   Valloc = 0;
}

inline void CPalette::setColor(int c, u_char r, u_char g, u_char b)
{
   if (c >= VnumColors) return;
   Vpalette[c][RED] = r;
   Vpalette[c][GREEN] = g;
   Vpalette[c][BLUE] = b;
}

inline void CPalette::getRGB(int c, u_char& r, u_char& g, u_char& b)
{
   if (c >= VnumColors)
      return;
   r = Vpalette[c][RED];
   g = Vpalette[c][GREEN];
   b = Vpalette[c][BLUE];
}


#endif
