// -*-c++-*-

/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * DisplayBox.h: DisplayBox class definition.  An abstract class for deriving
 * different kinds of display boxes from.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
// A class to display information in a box
#ifndef __DisplayBox__
#define __DisplayBox__

#include <stdio.h>
#include <SLList.h>
#include <String.h>

class DisplayBox {
public:
   DisplayBox()
   {
      UnderScreen = NULL;
      Width = Height = 0;
      X = Y = 0;
      Title = NULL;
   }
   
   virtual ~DisplayBox();

   int x() const { return X; }
   int y() const { return Y; }
   int width() const { return Width; }
   int height() const { return Height; }
   char *const title() const { return Title; }

protected:
   int X, Y;
   int Width, Height;
   const int BorderWidth = 8;	// width reserved for border
   SLList<String*> TextLines;
   char *Title;

   int WrapText(const char *const text);
   void DrawBorder(const char *const title = NULL) const;
   void PrintTextLines(int y1, int y2, bool centered) const;

   // Save screen under box.  Must be called before displaying box.
   void SaveUnder() const;
   
   // restore what was previously saved by SaveUnder. SaveUnder must have been
   // called first.
   void RestoreUnder() const;

private:
   // This is mutable so SaveUnder and RestoreUnder can be called from 
   // otherwise const functions.
   mutable char *UnderScreen;
};

#endif

