
/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * InfoBox.cc: InfoBox member functions.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
// Methods for the displaybox class

#include <vga.h>
#include <vgagl.h>
#include <stdio.h>
#include <string.h>
#include <Pix.h>

#include "Ui.h"
#include "InfoBox.h"

InfoBox::InfoBox(int x, int y, int width, int height, bool centered,
                      char *title, char *contents)
{
   X = x;
   Y = y;
   Width = width;
   Height = height;
   Centered = centered;
   if (title) {
      Title = new char[strlen(title) + 1];
      strcpy(Title, title);
   }

   if (contents)
      WrapText(contents);
}

InfoBox::~InfoBox()
{
   // currently does nothing
}

void InfoBox::setContents(const char *const contents)
{
   WrapText(contents);
}

void InfoBox::update() const
{
   int screeny;

   gl_fillbox(X + BorderWidth, Y + BorderWidth, Width - 2 * BorderWidth,
	      Height - 2 * BorderWidth, 0);

   if (TextLines.empty())
      return;

   if (Centered)		// center text vertically
   {
      int textheight = TextLines.length() * Ui::fontHeight();
      if (textheight < Height - 2 * BorderWidth) // if it fits
	 screeny = Y + (Height - textheight) / 2;
      else
	 screeny = Y + BorderWidth;
   }
   else
      screeny = Y + BorderWidth;
   PrintTextLines(screeny, (Y + Height) - BorderWidth, Centered);
}

void InfoBox::display() const
{
   gl_fillbox(X, Y, Width, Height, 0);
   DrawBorder(Title);
   update();
}


