
/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * Mouse.cc: Mouse member functions and data.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "Mouse.h"

// 8x8 cursor
unsigned char Mouse::DefaultCursor[] = {
   1, 1, 1, 1, 1, 1, 0, 0, 
   1, 2, 2, 1, 0, 0, 0, 0, 
   1, 2, 2, 1, 1, 0, 0, 0, 
   1, 1, 1, 2, 1, 1, 0, 0, 
   1, 0, 1, 1, 2, 1, 1, 0, 
   1, 0, 0, 1, 1, 2, 1, 1, 
   0, 0, 0, 0, 1, 1, 1, 0, 
   0, 0, 0, 0, 0, 1, 0, 0
};

/**  This mouse cursor looks better for 640x480x256 than the one above
// 11x19 cursor
unsigned char Mouse::DefaultCursor[] = {
   1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0
   1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
   1, 2, 2, 1, 0, 0, 0, 0, 0, 0, 0, 
   1, 2, 2, 2, 1, 0, 0, 0, 0, 0, 0, // 3
   1, 2, 2, 2, 2, 1, 0, 0, 0, 0, 0,
   1, 2, 2, 2, 2, 2, 1, 0, 0, 0, 0,
   1, 2, 2, 2, 2, 2, 2, 1, 0, 0, 0,
   1, 2, 2, 2, 2, 2, 2, 2, 1, 0, 0, // 7
   1, 2, 2, 2, 2, 2, 2, 2, 2, 1, 0,
   1, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1,
   1, 2, 2, 1, 2, 2, 1, 0, 0, 0, 0,
   1, 2, 1, 1, 2, 2, 1, 0, 0, 0, 0, // 11
   1, 1, 0, 1, 1, 2, 2, 1, 0, 0, 0,
   1, 0, 0, 0, 1, 2, 2, 1, 0, 0, 0,
   0, 0, 0, 0, 0, 1, 2, 2, 1, 0, 0,
   0, 0, 0, 0, 0, 1, 2, 2, 1, 0, 0, // 15
   0, 0, 0, 0, 0, 0, 1, 2, 2, 1, 0,
   0, 0, 0, 0, 0, 0, 1, 2, 2, 1, 0,
   0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0
};
*************************************/   
   
int Mouse::X;
int Mouse::Y;
int Mouse::oldX;
int Mouse::oldY;
int Mouse::Buttons;

bool Mouse::Ok = false;
bool Mouse::Alloca = false;
bool Mouse::Visible = false;

const int Mouse::MOUSEC_WIDTH = 8;
const int Mouse::MOUSEC_HEIGHT = 8;

SBitmap *Mouse::Cursor = 0;
unsigned char *Mouse::UnderCursor = 0;

int Mouse::init(int dark, int light, char *mousedev)
{
   Alloca = true;
   MakeDefault(dark, light);
   UnderCursor = new unsigned char[Cursor->size()];
   if (initMouse(mousedev))
      return -1;
   return 0;
}

int Mouse::init(SBitmap *b, char *mousedev)
{
   Alloca = false;
   Cursor = b;
   UnderCursor = new unsigned char[Cursor->size()];
   if (initMouse(mousedev))
      return -1;
   return 0;
}

int Mouse::init(char *mousedev)
{
   Alloca = false;
   if (initMouse(mousedev))
      return -1;
   return 0;
}

bool Mouse::update (void)
{
   if (mouse_update()) {
      oldX = X;
      oldY = Y;
      X = mouse_getx();
      Y = mouse_gety();
      Buttons = mouse_getbutton();

      if (Visible) {
	 RestoreUnder(oldX, oldY);
	 SaveUnder(X, Y);
	 DrawCursor(X, Y);
      }
      return true;
   } else
      return false;
}

void Mouse::setposition (int x, int y)
{
   mouse_setposition(x, y);
   oldX = X;
   oldY = Y;
   X = x;
   Y = y;
   if (Visible) {
      RestoreUnder(oldX, oldY);
      SaveUnder(X, Y);
      DrawCursor(X, Y);
   }
}

void Mouse::setVisible (bool v)
{
   if (v == true && Visible == false) {
      SaveUnder(X, Y);
      DrawCursor(X, Y);
   } else if (v == false && Visible == true)
      RestoreUnder(X, Y);
   Visible = v;
}

   // returns -1 on error, 0 on successful initialization of mouse
int Mouse::initMouse (char *mousedev)
{
   if (mouse_init(mousedev, vga_getmousetype(), MOUSE_DEFAULTSAMPLERATE))
      return -1;
   mouse_setxrange(0, WIDTH - 1);
   mouse_setyrange(0, HEIGHT - 1);
   mouse_setwrap(MOUSE_NOWRAP);
   setposition(WIDTH/2, HEIGHT/2);
   oldX = X = mouse_getx();
   oldY = Y = mouse_gety();
   Ok = true;
   return 0;
}

   
// creates the default mouse pointer from the 'DefaultCursor' template.
// 1s in the template are replaced with the dark color, 2s are replaced
// with light.
void Mouse::MakeDefault(int dark, int light)
{
   int size = MOUSEC_HEIGHT * MOUSEC_WIDTH;
   if (Cursor)
      delete Cursor;

   Cursor = new SBitmap(MOUSEC_WIDTH, MOUSEC_HEIGHT);
   unsigned char *buf = Cursor->map();
   for (int i = 0; i < size; i++) {
      switch (DefaultCursor[i]) {
      case 0:
	 buf[i] = 0; break;
      case 1:
	 buf[i] = dark; break;
      case 2:
	 buf[i] = light; break;
      }
   }
}
