// -*-c++-*-

/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * Mouse.h: Mouse class definition.  This is a static class interface to 
 * vgamouse.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// File   : Mouse.h
// Name   : gamelib1.0
// Changed: Sun Jul 15 14:22:17 1995

#ifndef __Mouse__
#define __Mouse__

#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include "SBitmap.h"


// These defines are for the alternate Cursor in Mouse.cc
//#define MOUSEC_WIDTH 11
//#define MOUSEC_HEIGHT 19

// This class has all static member functions and variables.  So there is no
// point in decalaring an instance of it.  I did this because as far as I know
// There can only be one mouse running at a time.
class Mouse {
public:
				// use default pointer with fg/bg colors
   static int init (int dark, int light, char *mousedev = "/dev/mouse");
				// use b as the mouse pointer bitmap
   static int init (SBitmap *b, char *mousedev = "/dev/mouse");
				// don't use a visible pointer
   static int init (char *mousedev = "/dev/mouse"); 
   static void close(void);

   static bool update (void);	// updates mouse, returns false if no updates
   static bool ok();		// true if mouse inititialized
   static int x (void);		// returns mouse x coord
   static int y (void);		// returns mouse y coord
   
   static bool leftb (void);	// returns true if left button pushed
   static bool middleb (void);	// true if middle button pushed
   static bool rightb (void);	// true if right button pushed
   
   static bool left (void);	// true if mouse moved left since last update
   static bool right (void);	// true if mouse moved right 
   static bool up (void);	// true if mouse moved up ...
   static bool down (void);	// true if mouse moved down

   static void RestoreUnder (int x, int y);
   static void SaveUnder (int x, int y);
   static void DrawCursor (int x, int y);

    // All of these settings have default values
   static void setVisible(bool v); // if true, mouse pointer is visible
   static void setBitmap(SBitmap *b); // set mouse pointer to use b
   static void setposition(int x, int y); // set mouse position to x/y
   static void setxrange(int x1, int x2); // set horizontal range 
   static void setyrange(int y1, int y2); // set vertical range
   static void setscale(int s);	   // scale that mouse coords be divided by
   // argument to setwrap can be MOUSE_NOWRAP, MOUSE_WRAPX, MOUSE_WRAPY,
   // or MOUSE_WRAP, which sets both WRAPX and WRAPY
   static void setwrap(int w); // set whether mouse wraps around screen

private:
   static int X;		// current mouse x
   static int Y;		// current mouse y
   static int oldX;		// old mouse x
   static int oldY;		// old mouse y
   static int Buttons;		// state of mouse buttons
   static const int MOUSEC_WIDTH;
   static const int MOUSEC_HEIGHT;

   static bool Alloca;		// true if mem was allocate by self
   static bool Visible;		// true if mouse cursor is visible
   static bool Ok;		// true if Mouse has been initialized

   static SBitmap *Cursor;	// mouse cursor bitmap
   static unsigned char *UnderCursor; // save region under mouse pointer
   static unsigned char DefaultCursor[]; 

   static int initMouse (char *mousedev); // initialize mouse defaults
   static void MakeDefault (int dark, int light); // make default cursor
};

inline bool Mouse::ok(void)
{
   return Ok;
}

inline int Mouse::x(void)
{
   return X;
}

inline int Mouse::y(void)
{
   return Y;
}

inline bool Mouse::leftb(void)
{
   return (Buttons & MOUSE_LEFTBUTTON) ? true : false;
}

inline bool Mouse::middleb(void)
{
   return (Buttons & MOUSE_MIDDLEBUTTON) ? true : false;
}

inline bool Mouse::rightb(void)
{
   return (Buttons & MOUSE_RIGHTBUTTON) ? true : false;
}

inline bool Mouse::left(void)
{
   return (X < oldX) ? true : false;
}

inline bool Mouse::right(void)
{
   return (X > oldX) ? true : false;
}

inline bool Mouse::up(void)
{
   return (Y < oldY) ? true : false;
}

inline bool Mouse::down(void)
{
   return (Y > oldY) ? true : false;
}

inline void Mouse::RestoreUnder (int x, int y)
{
   gl_putbox(x, y, Cursor->width(), Cursor->height(), UnderCursor);
}

inline void Mouse::SaveUnder (int x, int y)
{
   gl_getbox(x, y, Cursor->width(), Cursor->height(), UnderCursor);
}

inline void Mouse::DrawCursor (int x, int y)
{
   Cursor->put(x, y);
}



inline void Mouse::setBitmap (SBitmap *b)
{
   if (Alloca)
      delete Cursor;
   delete [] UnderCursor;
   Cursor = b;
   UnderCursor = new unsigned char[Cursor->size()];
}

inline void Mouse::setxrange(int x1, int x2)
{
   mouse_setxrange(x1, x2);
}

inline void Mouse::setyrange(int y1, int y2)
{
   mouse_setyrange(y1, y2);
}

inline void Mouse::setscale(int s)
{
   mouse_setscale(s);
}

inline void Mouse::setwrap(int w)
{
   mouse_setwrap(w);
}

inline void Mouse::close(void)
{
   mouse_close();
   if (Alloca) {
      delete Cursor;
   }
   delete [] UnderCursor;
}

#endif				// __Mouse__

