
/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * MsgBox.cc: MsgBox member functions.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <vga.h>
#include <vgagl.h>
#include <stdio.h>
#include <string.h>

#include "MsgBox.h"
#include "Button.h"
#include "Mouse.h"
#include "Ui.h"
#include "Keyboard.h"
#include "Colors.h"

MsgBox::MsgBox(int x, int y, int width, int height, bool centered,
	       char *contents, char *title)
{
   X = x;
   Y = y;
   Width = width;
   Height = height;
   Centered = centered;
   if (contents)
      WrapText(contents);

   if (title)
   {
      Title = new char[strlen(title) + 1];
      strcpy(Title, title);
   }

   OkBn = new Button(0, 0, 40, "Ok");
}

MsgBox::MsgBox(int x, int y, bool centered, char *contents, char *title)
{
   X = x;
   Y = y;
   if (contents)
      Width = (WrapText(contents) * Ui::fontWidth()) + 2 * BorderWidth;
   OkBn = new Button(0, 0, 40, "Ok");
   Height = (TextLines.length() * Ui::fontHeight()) + (2 * BorderWidth)
      + OkBn->height() + 16;

   Centered = centered;
   if (title)
   {
      Title = new char[strlen(title) + 1];
      strcpy(Title, title);
   }
}

MsgBox::MsgBox(bool centered, char *contents, char *title)
{
   if (contents)
      Width = (WrapText(contents) * Ui::fontWidth()) + 2 * BorderWidth;
   OkBn = new Button(0, 0, 40, "Ok");
   Height = (TextLines.length() * Ui::fontHeight()) + (2 * BorderWidth)
      + OkBn->height() + 16;

   X = (WIDTH - Width) / 2;
   Y = (HEIGHT - Height) / 2;

   Centered = centered;
   if (title)
   {
      Title = new char[strlen(title) + 1];
      strcpy(Title, title);
   }
}

MsgBox::~MsgBox()
{
   delete OkBn;
}

void MsgBox::display() const
{   
   gl_fillbox(X, Y, Width, Height, 0);

   DrawBorder(Title);

   const int max_textheight = Height - (2 * BorderWidth + OkBn->height() + 16);
   int screeny;
   if (Centered)
   {
      const int textheight = TextLines.length() * Ui::fontHeight();
      if (textheight <= max_textheight) // if it fits
	 screeny = Y + BorderWidth + (max_textheight - textheight) / 2;
      else
	 screeny = Y + BorderWidth;
   }
   else
      screeny = Y + BorderWidth;
   
   PrintTextLines(screeny, Y + BorderWidth + max_textheight, Centered);

   gl_hline(X, Y + BorderWidth + max_textheight + 8, X + Width - 1, Clr::WHITE);

   OkBn->display(X + (Width - OkBn->width()) / 2,
		 (Y + Height) - (BorderWidth + OkBn->height()));
}

void MsgBox::activate() const
{
   if (Mouse::ok())
      Mouse::setVisible(false);
   SaveUnder();			// save under box
   
   display();     // display box
   
   if (Mouse::ok())
      Mouse::setVisible(true);

   bool quit = false;
   while (!quit)
   {
      if (Keyboard::update())
      {
	 if (Keyboard::keypressed(SCANCODE_ESCAPE) ||
	     Keyboard::keypressed(SCANCODE_ENTER))
	    quit = true;
      }

      if (Mouse::ok() && Mouse::update() && Mouse::leftb())
      {
	 if (OkBn->inside(Mouse::x(), Mouse::y()))
	    quit = true;
      }
   }  // while

   if (Mouse::ok())
      Mouse::setVisible(false);
   RestoreUnder();		// restore screen under Box
   if (Mouse::ok())
      Mouse::setVisible(true);
}

