#include <vga.h>
#include <vgagl.h>
#include <stdlib.h>
#include <stdio.h>
#include <GetOpt.h>
#include <iostream.h>
#include <fstream.h>

#include "Mouse.h"
#include "Button.h"
#include "Ui.h"
#include "Keyboard.h"

int loadfont(char *fontfile, int width, int height)
{
   ifstream Fontf(fontfile);
   if (Fontf.fail()) {
      cerr << "Failed to open " << fontfile << endl;
      return -1;
   }
   char *buf = new char[(width * height/8)*256];
   Fontf.read(buf, (width*height/8)*256);
   if (Fontf.fail()) {
      cerr << "Failed to read fontfile " << fontfile << endl;
      delete [] buf;
      return -1;
   }
   char *expanded = new char[width * height * BYTESPERPIXEL * 256];
   gl_expandfont(width, height, 255, buf, expanded);
   Ui::setFont(expanded, width, height);
   delete [] buf;
   return 0;
}
   
void printalpha(int width, int height)
{
   int x = 0, y = 0;
   char buf[2];
   for (int i = 0; i < 256; i++) {
      sprintf(buf, "%c", i);
      gl_write(x, y, buf);
      x += width;
      if (x > (WIDTH-width)) {
         x = 0;
         y += height;
      }
   }
}
      

int main(int argc, char **argv)
{
   int opt, vgamode = G320x200x256;
   bool font = false;
   GetOpt getopt(argc, argv, "m:f");

   while ((opt = getopt()) != EOF) {
      switch (opt) {
      case 'm':
         vgamode = vga_getmodenumber(getopt.optarg);
         if (vgamode < 0) {
            cerr << "Unknown vgamode :" << getopt.optarg << endl;
            exit(1);
         }
         break;
      case 'f':
         font = true;
         break;
      case '?':
         exit(1);
      } // switch
   }  // while
   
   Ui::init(vgamode);
   Ui::drawToPhysical();
   if (font)
      loadfont("/usr/games/lib/Vga16font8x16", 8, 16);
   Keyboard::init();
   Mouse::init(1, 255);	// 137 = red, 241 = yellow
   Button quitb(250, 50, "Quit");

   gl_clearscreen(222);
 
   Mouse::setVisible(true);
   Keyboard::update();
   quitb.display();

   while (!Keyboard::keypressed(SCANCODE_Q)) {
      if (Mouse::update() && Mouse::leftb()) {
	 if (quitb.inside(Mouse::x(), Mouse::y()))
	    break;
      }
      Keyboard::update();
   }

   Keyboard::restore();
//   Ui::input(100, 100, "Enter your name ", buf, 64);
//   Ui::message(108, buf);

   printalpha(8, 16);
   while(vga_getkey() != 'q');

   Mouse::close();
   Ui::restore();
}
