#include <vga.h>
#include <vgagl.h>
#include <GetOpt.h>
#include <stdlib.h>
#include <iostream.h>

#include "Ui.h"
#include "Keyboard.h"
#include "Menu.h"
#include "Colors.h"
#include "Mouse.h"

int main(int argc, char **argv)
{
   GetOpt getopt(argc, argv, "x:y:w:h:m:");
   int width = 0;
   int height = 0;
   int x = 40;
   int y = 40;
   int vgamode = G640x480x256;
   int opt;
   Menu *MyMenu;

   while ((opt = getopt()) != -1) {
      switch (opt) {
      case 'x':
	 x = atoi(getopt.optarg);
	 break;
      case 'y':
	 y = atoi(getopt.optarg);
	 break;
      case 'w':
	 width = atoi(getopt.optarg);
	 break;
      case 'h':
	 height = atoi(getopt.optarg);
	 break;
      case 'm':
	 vgamode = vga_getmodenumber(getopt.optarg);
	 if (vgamode < 0) {
	    cerr.form("%s: Unknown vgamode '%s'\n", argv[0], getopt.optarg);
	    exit(1);
	 }
	 break;
      case '?':
	 cerr.form("%s: options are --
-w <width>    width of menu
-h <height>   height
-m <mode>     vga mode
\n", argv[0]);
	 exit(1);
      }
   }

   Ui::init(vgamode);
   Keyboard::init();
   Mouse::init(Clr::BLACK, Clr::WHITE);

   gl_clearscreen(Clr::GREEN);

   #define DOG 1
   #define CAT 2
   #define PIG 3
   #define REMOVE 4
   #define QUIT 5
   

   if (width == 0 || height == 0)
      MyMenu = new Menu("Menu", "Auto Dimensions");
   else
      MyMenu = new Menu(x, y, width, height, "Menu", "Given Dimensions");

   MyMenu->add("1. Choose a dog", 'd', DOG);
   MyMenu->add("2. Choose a cat", 'c', CAT);
   MyMenu->add("3. Choose a pig", 'p', PIG);
   MyMenu->add("4. Remove an item", 'r', REMOVE);
   MyMenu->add("5. Quit program", 'q', QUIT);

   Mouse::setVisible(true);

   int command;
   char buf[20];
   while ((command = MyMenu->activate()) != QUIT) {
      gl_setwritemode(WRITEMODE_OVERWRITE);
      switch (command) {
      case DOG:
	 gl_write(20, HEIGHT - 20, "You chose dog");
	 break;
      case CAT:
	 gl_write(20, HEIGHT - 20, "You chose cat");
	 break;
      case PIG:
	 gl_write(20, HEIGHT - 20, "You chose pig");
	 break;
      case -1:
	 gl_write(20, HEIGHT - 20, "You cancelled");
	 break;
      case REMOVE:
	 Keyboard::restore();
	 Ui::input(150, HEIGHT - 20, "Enter a number ", buf, 20);
	 Keyboard::init();
	 MyMenu->remove(atoi(buf));
	 gl_clearscreen(Clr::GREEN);
	 break;
      }
   }

   delete MyMenu;
   Keyboard::restore();
   Ui::restore();
   Mouse::close();
}
	 
	 



