// -*-c++-*-
/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * tetris.h: header file for main program.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __tetris__
#define __tetris__

#include <fstream.h>
#include <SLList.h>

#include <Button.h>
#include <HiScore.h>
#include <InfoBox.h>
#include "Board.h"


typedef enum {
   NO_CMD,
   QUIT_CMD,
   PLAY_CMD,
   SCORES_CMD,
   LEVEL_CMD,
   TYPE_CMD,
   PAUSE_CMD,
   MENU_CMD,
   NEXTP_CMD,		// toggle next piece 
   BOARDSH_CMD		// toggle board shadow
} commands;

class GameInfo {
public:
   GameInfo() 
   {
      Level = Points = Lines = 0;
      State = wait;
      boardshadow = nextpiece = true;
   }
   
   int setLevel(int level) 
   { 
      if (level < 0 || level > 16)
         return -1;
      else 
         Level = level; 
      return 0;
   }
   
   void incLevel(int by = 1) 
   { 
      Level += by; 
      if (Level > 16)
         Level = 16;
   }

   void decLevel(int by = 1) 
   { 
      if (Level < by)
         Level = 0;
      else
         Level -= by;
   }

   void setPoints(Score points) { Points = points; }
   void incPoints(int by = 1) { Points += by; }
   void decPoints(int by = 1) 
   { 
      if (Points < by)
	 Points = 0;
      else
         Points -= by;
   }

   void setLines(int lines) { Lines = lines; }
   void incLines(int by = 1) { Lines += by; }
   void decLines(int by = 1) 
   { 
      Lines -= by;
      if (Lines < 0)
         Lines = 0;
   }

   void toggleBoardShadow() { boardshadow = !boardshadow; }
   void toggleNextPiece() { nextpiece = !nextpiece; }

   enum PlayState { 
      pause, 
      play, 
      wait };
   void setState(PlayState s) { State = s; }

   enum GameType {
      game1 = 1,
      game2 = 2,
      game3 = 3 };
      
   int setType(int t) 
   { 
      if (t >= game1 && t <= game3) 
      {
         Type = (GameType)t;
         return 0;
      } 
      else
         return -1;
   }

   bool Type1() const { return Type == game1; }
   bool Type2() const { return Type == game2; }
   bool Type3() const { return Type == game3; }

   int setHeight(int h) 
   {
      if (h > 0 && h < 10) 
      {
         Height = h;
         return 0;
      } 
      else
         return -1;
   }

   int height() const { return Height; }
   int level() const { return Level; }
   Score points() const { return Points; }
   int lines() const { return Lines; }
   
   bool pauseState() const { return (State == pause) ? true : false; }
   bool playState() const { return (State == play) ? true : false; }
   bool waitState() const { return (State == wait) ? true : false; }
   bool BoardShadow() const { return boardshadow; }
   bool NextPiece() const { return nextpiece; }

private:
   int Level;		// current play level
   Score Points;	// current points
   int Lines;		// number of lines erased
   PlayState State;
   GameType Type;	// game type
   int Height;		// height for game2
   bool boardshadow;	// if true, display board shadow
   bool nextpiece;	// if true, display next piece
};

extern Board *MainBoard, *NextPiece, *BoardShadow;
extern SLList<Button*> ButtonList;
extern GameInfo game_stats;
extern InfoBox *StatusBox, *HelpBox;

// functions
extern void drawScreen(void);
extern void setHelp(void);
extern int setGameKeyboard(void);
extern void updateStatus(void);
extern void CleanUp(void);
extern void toggleNextPiece(void);
extern void toggleBoardShadow(void);

#ifdef DEBUG
extern ofstream debugf;
#endif

 
#endif // __tetris__
