/* Tetris v0.4 - Tetris game for VGA PCs running Linux
 * Copyright (C) 1995 Adam P. Jenkins
 *
 * utils.cc: Miscellaneous other functions for tetris game.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <fstream.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>

#include <Ui.h>
#include <Colors.h>

#include "mouse_cursor.h"
#include "utils.h"

// prints a string at x/y, at 'mag' the size of the current font size
void PrintMag(int x, int y, float mag, char *s)
{
   if (s == NULL)
      return;

   int w = strlen(s) * Ui::fontWidth();
   int h = Ui::fontHeight();
   int ew = (int)(w * mag);	// expanded width and height
   int eh = (int)(h * mag);

   // first create a bitmap of the string at normal font size
   char *buf = new char[w * h];
   bzero(buf, w * h);
   gl_setcontextvirtual(w, h, BYTESPERPIXEL, BITSPERPIXEL, buf);
   gl_setwritemode(WRITEMODE_MASKED);
   gl_write(0, 0, s);

   // use gl_scalebox to scale the bitmap to the new size
   char *ebuf = new char[ew * eh];	// expand bitmap
   gl_scalebox(w, h, buf, ew, eh, ebuf);

   Ui::drawToPhysical();
   gl_putboxmask(x, y, ew, eh, ebuf);	// print expanded version
   delete [] buf;
   delete [] ebuf;
}

// Load a font stored as a bit-per-pixel bitmap.
int loadFont(char *fontfile, int width, int height)
{
   ifstream Fontf(fontfile);
   if (Fontf.fail()) 
   {
      cerr << "Failed to open " << fontfile << endl;
      return -1;
   }
   
   int fontbytes = (width * height/8) * 256;
   char *buf = new char[fontbytes];

   Fontf.read(buf, fontbytes);
   if (Fontf.fail()) 
   {
      cerr << "Failed to read fontfile " << fontfile << endl;
      delete [] buf;
      return -1;
   }
   
   unsigned char *expanded = 
      new unsigned char[width * height * BYTESPERPIXEL * 256];
   gl_expandfont(width, height, 255, buf, expanded);
   Ui::setFont(expanded, width, height);

   delete [] buf;
   return 0;
}

// Creates the mouse cursor defined in mouse_cursor.h. Returns NULL if
// unsuccessful, otherwise SBitmap *.
SBitmap *MakeMouseCursor(int dark, int light)
{
   SBitmap *cursor = new SBitmap(MOUSE_WIDTH, MOUSE_HEIGHT);
   if (!cursor)
      return NULL;
   int size = MOUSE_WIDTH * MOUSE_HEIGHT;
   unsigned char *buf = cursor->map();

   for (int i = 0; i < size; i++) 
   {
      switch (mouse_cursor[i]) {  // mouse_cursor is defined in mouse_cursor.h
      case 0:
         buf[i] = 0;  break;
      case 1:
         buf[i] = dark; break;
      case 2:
         buf[i] = light; break;
      }
   }

   return cursor;
}

// this function taken from Brad Pitzel's Sasteroids game
// Reads name for the High Score list.
String getHighScoreName()
{
   char buf[31], *ptr;
   for(int i = 0; i < 21; i++) 
      buf[i] = ' ';    

   // default to user's login name
   ptr = getlogin();
   if (ptr) 
      strcpy(buf, ptr);
   else
      buf[0] = '\0';
   
   Ui::drawToPhysical();

   gl_clearscreen(0);

   gl_write( 20, 30, "Whooohoo!");
   gl_write( 20, 40, "You made the high score list.");

   gl_write( 20, 60, "Please enter your name :" );

   Ui::setFontColor(Clr::RED);
   Ui::input( 20, 80, buf, 20 );
   Ui::setFontColor(Clr::WHITE);

   return String(buf);
}

// this func taken from Brad Pitzel's sasteroids game
// show high scores screen
void showScores( HiScore &s )
{
   int i;
   char buf[10];
   String line;

   gl_clearscreen(66);

   int x = (WIDTH - 320)>>1;
   int y = (HEIGHT - 200)>>1;
	   
   gl_setwritemode(WRITEMODE_MASKED);
   gl_write( x+120, y+0,"HIGH SCORES");

   for(i=0; i<HiScoreListSize; i++)
   {
      sprintf(buf,"%d.",i+1);
      gl_write( x+40, y+40+15*i,buf);

      line = s[i];
      gl_write( x+70, y+40+15*i, (char *)(const char*)line );
   }

   gl_write( x+104, y+190,"(Press any key)");
   gl_setwritemode(WRITEMODE_OVERWRITE);
   getchar();
}


